local tomQuest2 = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")

-----------------------
--- Library Loading ---
-----------------------
local LCo = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Config")
local LSM = LibStub("LibSharedMedia-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName = CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName
local pairs, type, sub, tonumber, format, unpack, IsAddOnLoaded = pairs, type, string.sub, tonumber, string.format, unpack, IsAddOnLoaded

-------------------------
--- Main Option table ---
-------------------------
local raidModeValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
}
local raidModeTrackerValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
	["hiddenInRaid"] = LCo["Hidden once entering a raid"],
	["hiddenInRaidInstance"] = LCo["Hidden once entering a raid instance (will work in pvp and outdoor raids)"],
}
local moneySeparators = {
	["dot"] = LCo["Dot"],
	["symbol"] = LCo["Symbol"],
	["texture"] = LCo["Texture"],
}
local questLevelTag = {
	["questLevel"] = LCo["Quest level"],
	["weightedQuestLevel"] = LCo["Weighted quest level"],
	["Hide"] = LCo["Hide"],
}
local zSorting = {
	["levelA"] = LCo["Sort zones based on level (ascending)"],
	["levelD"] = LCo["Sort zones based on level (descending)"],
	["nameA"] = LCo["Sort zones based on zone name (ascending)"],
	["nameD"] = LCo["Sort zones based on zone name (descending)"],
}
local zwlPosition = {
	[0] = LCo["Zones without level first"],
	[100] = LCo["Zones without level last"]
}
local qSorting = {
	["weightedA"] = LCo["Sort quests based on weighted level (ascending)"],
	["weightedD"] = LCo["Sort quests based on weighted level (descending)"],
	["levelA"] = LCo["Sort quests based on quest level (ascending)"],
	["levelD"] = LCo["Sort quests based on quest level (descending)"],
	["nameA"] = LCo["Sort quests based on name (ascending)"],
	["nameD"] = LCo["Sort quests based on name (descending)"],
}
local function hideCustomColorMode()
	if tomQuest2.db.profile.customColorMode then
		return true
	else
		return false
	end
end
local function computeQuestsObjectivesStatus()
	for qid, qoData in pairs (tomQuest2.questsObjectives) do
		for oid in pairs (qoData) do
			tomQuest2:updateQuestObjectiveCells(qid, oid)
		end
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function normalizeColorComponent(component)
	if component > 1 then return 1 elseif component < 0 then return 0 else return component end
end
local function computeFirstFunction(r0, g0, b0, rh, gh, bh)
	tomQuest2.db.profile.rj1 = r0
	tomQuest2.db.profile.gj1 = g0
	tomQuest2.db.profile.bj1 = b0
	tomQuest2.db.profile.ri1 = 2 * (rh - r0)
	tomQuest2.db.profile.gi1 = 2 * (gh - g0)
	tomQuest2.db.profile.bi1 = 2 * (bh - b0)
end
local function computeSecondFunction(rh, gh, bh, r1, g1, b1)
	tomQuest2.db.profile.rj2 = 2 * rh - r1
	tomQuest2.db.profile.gj2 = 2 * gh - g1
	tomQuest2.db.profile.bj2 = 2 * bh - b1
	tomQuest2.db.profile.ri2 = 2 * (r1 - rh)
	tomQuest2.db.profile.gi2 = 2 * (g1 - gh)
	tomQuest2.db.profile.bi2 = 2 * (b1 - bh)
end

local function optGetter(info)
	return tomQuest2.db.profile[info[#info]]
end
local function optSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
end
local function optMGetter(info)
	return tomQuest2.db.profile.modules[info[#info]]
end
local function optMSetter(info, value)
	tomQuest2.db.profile.modules[info[#info]] = value
	if value then
		tomQuest2:enableModule(info[#info])
	else
		tomQuest2:disableModule(info[#info])
	end
end
local function optRMGetter(info)
	return tomQuest2.db.profile.raidModules[info[#info]]
end
local function optRMSetter(info, value)
	local oldState = tomQuest2.db.profile.raidModules.enabled
	tomQuest2.db.profile.raidModules[info[#info]] = value
	if tomQuest2.db.profile.raidModules.enabled then
		tomQuest2:RAID_ROSTER_UPDATE(nil, true)
	elseif oldState then
		for k, state in tomQuest2.db.profile.modules do
			if state and not tomQuest2.moduleState[k] then
				tomQuest2:enableModule(k)
				tomQuest2:Print(format(LCo["%s module is now enabled"], k))
			end
			if tomQuest2.db.profile.raidModules[k] == "hiddenInRaid" or tomQuest2.db.profile.raidModules[k] == "hiddenInRaidInstance" then
				if k == "questsTracker" then
					tomQuest2:tQ2toggleQuestTracker(1)
				elseif k == "achievementsTracker" then
					tomQuest2:tQ2toggleAchievementsTracker(1)
				end
			end
		end
	end
end
local function optPHSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	if tomQuest2.numPartyMembers == 0 then return end
	local partyCategory = tomQuest2.PARTYQUESTSZONE
	if value then
		for qid in pairs(tomQuest2.quests) do
			if not tomQuest2.quests[qid].playerStatus then
				local category = tomQuest2.quests[qid].zone
				tomQuest2.questsByCategory[category][qid] = nil
				if not next(tomQuest2.questsByCategory[category]) then
					tomQuest2.questsCategories[category] = nil
					tomQuest2.questsByCategory[category] = nil
					tomQuest2:removeQuestCategoryCells(category)
				end
				if not tomQuest2.questsCategories[partyCategory] then
					tomQuest2:addQuestCategory(partyCategory)
				end
				tomQuest2.questsByCategory[partyCategory][qid] = true
			end
		end
		tomQuest2:categoriesSort()
		tomQuest2:questsSort()
		tomQuest2:setQuestsTrackerLayout()
	elseif tomQuest2.questsByCategory[partyCategory] then
		for qid in pairs(tomQuest2.questsByCategory[partyCategory]) do
			local zone = tomQuest2.quests[qid].zone
			if not tomQuest2.questsCategories[zone] then
				tomQuest2:addQuestCategory(zone)
			end
			tomQuest2.questsByCategory[zone][qid] = true
			if zone == tomQuest2.localClass then
				tomQuest2.quests[qid].class = true
			end
		end
		tomQuest2.questsCategories[partyCategory] = nil
		tomQuest2.questsByCategory[partyCategory] = nil
		tomQuest2:categoriesSort()
		tomQuest2:questsSort()
		tomQuest2:setQuestsTrackerLayout()
	end
end
local function optHideBTr(info, value)
	tomQuest2.db.profile[info[#info]] = value
	tomQuest2:hideShowDefaultBlizTracker()
end
local function specialItemMacro(info, value)
	local index = GetMacroIndexByName("tomQuest2_SI")
	if index == 0 then
		index = CreateMacro("tomQuest2_SI", 1, "", 0)
		PickupMacro(index)
	else
		EditMacro(index, "tomQuest2_SI", 1, "")
	end
end
local function optWQLSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	tomQuest2:computeQuestsLevel()
	tomQuest2:questsSort()
	tomQuest2:setQuestsTrackerLayout()
end
local function optTagSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	tomQuest2:computeQuestsTag()
end
local function optCCFSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	computeQuestsObjectivesStatus()
end
local function halfColorGetter(info)
	return normalizeColorComponent(0.5*tomQuest2.db.profile.ri1 + tomQuest2.db.profile.rj1), normalizeColorComponent(0.5*tomQuest2.db.profile.gi1 + tomQuest2.db.profile.gj1), normalizeColorComponent(0.5*tomQuest2.db.profile.bi1 + tomQuest2.db.profile.bj1)
end
local function fullColorGetter(info)
	return normalizeColorComponent(tomQuest2.db.profile.rj3), normalizeColorComponent(tomQuest2.db.profile.gj3), normalizeColorComponent(tomQuest2.db.profile.bj3)
end
local function zeroColorGetter(info)
	return normalizeColorComponent(tomQuest2.db.profile.rj0) , normalizeColorComponent(tomQuest2.db.profile.gj0), normalizeColorComponent(tomQuest2.db.profile.bj0)
end
local function zeroColorSetter(info, arg1, arg2, arg3)
	tomQuest2.db.profile.rj0 = arg1
	tomQuest2.db.profile.gj0 = arg2
	tomQuest2.db.profile.bj0 = arg3
	local rh, gh, bh = halfColorGetter()
	computeFirstFunction(arg1, arg2, arg3, rh, gh, bh)
	computeQuestsObjectivesStatus()
end
local function halfColorSetter(info, arg1, arg2, arg3)
	local r0, g0, b0 = zeroColorGetter()
	computeFirstFunction(r0, g0, b0, arg1, arg2, arg3)
	local r1, g1, b1 = fullColorGetter()
	computeSecondFunction(arg1, arg2, arg3, r1, g1, b1)
	computeQuestsObjectivesStatus()
end
local function fullColorSetter(info, arg1, arg2, arg3)
	local rh, gh, bh = halfColorGetter()
	computeSecondFunction(rh, gh, bh, arg1, arg2, arg3)
	tomQuest2.db.profile.rj3 = arg1
	tomQuest2.db.profile.gj3 = arg2
	tomQuest2.db.profile.bj3 = arg3
	computeQuestsObjectivesStatus()
end
local function setDefaultColor()
	tomQuest2.db.profile.rj0, tomQuest2.db.profile.gj0, tomQuest2.db.profile.bj0 = 1, 0, 0
	tomQuest2.db.profile.ri1, tomQuest2.db.profile.rj1, tomQuest2.db.profile.gi1, tomQuest2.db.profile.gj1, tomQuest2.db.profile.bi1, tomQuest2.db.profile.bj1 = 0, 1, 2, 0, 0 ,0
	tomQuest2.db.profile.ri2, tomQuest2.db.profile.rj2, tomQuest2.db.profile.gi2, tomQuest2.db.profile.gj2, tomQuest2.db.profile.bi2, tomQuest2.db.profile.bj2 = -2, 2, 0, 1, 0, 0
	tomQuest2.db.profile.rj3, tomQuest2.db.profile.gj3, tomQuest2.db.profile.bj3 = 0, 1, 0
	computeQuestsObjectivesStatus()
end
local function hexColorGetter(info)
	return tonumber(sub(tomQuest2.db.profile[info[#info]], 1, 2), 16) / 255, tonumber(sub(tomQuest2.db.profile[info[#info]], 3, 4), 16) / 255, tonumber(sub(tomQuest2.db.profile[info[#info]], 5, 6), 16) / 255
end
local function hexColorSetter(info, arg1, arg2, arg3)
	tomQuest2.db.profile[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
	tomQuest2:updateQuestCounter()
end
local function sortSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	tomQuest2:categoriesSort()
	tomQuest2:questsSort()
	tomQuest2:setQuestsTrackerLayout()
end
local function zwlSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	for zone, zData in pairs(tomQuest2.questsCategories) do
		if zData.minLevel == "" then
			tomQuest2.questsCategories[zone].sMinLevel = tomQuest2.db.profile.zoneWithoutLvlPos
		end
		if zData.maxLevel == "" then
			tomQuest2.questsCategories[zone].sMaxLevel = tomQuest2.db.profile.zoneWithoutLvlPos
		end
	end
	tomQuest2:categoriesSort()
	tomQuest2:setQuestsTrackerLayout()
end

local options = {
	type = "group",
	childGroups = "tab",
	name = "tomQuest2",
	get = optGetter,
	set = optSetter,
	args = {
		modules = {
			type = "group",
			name = LCo["Modules"],
			get = optMGetter,
			set = optMSetter,
			order = 1,
			args = {
				syncParty = {
					type = "toggle",
					name = LCo["Party syncing"],
					width = "double",
					order = 1,
				},
				achievementsTracker = {
					type = "toggle",
					name = LCo["Achievements tracker"],
					width = "double",
					order = 2,
				},
				questsTracker = {
					type = "toggle",
					name = LCo["Quests tracker"],
					width = "double",
					order = 3,
				},
				questsTooltip = {
					type = "toggle",
					name = LCo["Quests tooltip"],
					width = "double",
					order = 4,
				},
				dailyTooltip = {
					type = "toggle",
					name = LCo["Daily tooltip"],
					width = "double",
					order = 5,
				},
				informations = {
					type = "toggle",
					name = LCo["Information"],
					width = "double",
					order = 6,
				},
				notifications = {
					type = "toggle",
					name = LCo["Notifications"],
					width = "double",
					order = 7,
				},
				soundNotifications = {
					type = "toggle",
					name = LCo["Sound notifications"],
					width = "double",
					order = 8,
				},
				tagQuestLog = {
					type = "toggle",
					name = LCo["Tag quests log"],
					width = "double",
					order = 9,
				},
				questsGivers = {
					type = "toggle",
					name = LCo["Quests givers"],
					width = "double",
					order = 10,
				},
				tooltips = {
					type = "toggle",
					name = LCo["Tooltips"],
					width = "double",
					order = 11,
				},
				achievementsTradeSkills = {
					type = "toggle",
					name = LCo["Achievements trade skills"],
					width = "double",
					order = 12,
				},
				announce = {
					type = "toggle",
					name = LCo["Announce"],
					width = "double",
					order = 13,
				},
			},
		},
		raidMode = {
			type = "group",
			name = LCo["Raid mode"],
			get = optRMGetter,
			set = optRMSetter,
			order = 2,
			args = {
				enabled = {
					type = "toggle",
					name = LCo["Raid mode"],
					width = "full",
					order = 1,
				},
				raidModeDesc = {
					type = "description",
					name = LCo["Define module state when in a raid"],
					order = 2,
				},
				syncParty = {
					type = "select",
					values = raidModeValues,
					name = LCo["Party syncing"],
					width = "double",
					order = 3,
				},
				achievementsTracker = {
					type = "select",
					values = raidModeTrackerValues,
					name = LCo["Achievements tracker"],
					width = "double",
					order = 4,
				},
				questsTracker = {
					type = "select",
					values = raidModeTrackerValues,
					name = LCo["Quests tracker"],
					width = "double",
					order = 5,
				},
				questsTooltip = {
					type = "select",
					values = raidModeValues,
					name = LCo["Quests tooltip"],
					width = "double",
					order = 6,
				},
				dailyTooltip = {
					type = "select",
					values = raidModeValues,
					name = LCo["Daily tooltip"],
					width = "double",
					order = 7,
				},
				informations = {
					type = "select",
					values = raidModeValues,
					name = LCo["Information"],
					width = "double",
					order = 8,
				},
				notifications = {
					type = "select",
					values = raidModeValues,
					name = LCo["Notifications"],
					width = "double",
					order = 9,
				},
				soundNotifications = {
					type = "select",
					values = raidModeValues,
					name = LCo["Sound notifications"],
					width = "double",
					order = 10,
				},
				tagQuestLog = {
					type = "select",
					values = raidModeValues,
					name = LCo["Tag quests log"],
					width = "double",
					order = 11,
				},
				questsGivers = {
					type = "select",
					values = raidModeValues,
					name = LCo["Quests givers"],
					width = "double",
					order = 12,
				},
				tooltips = {
					type = "select",
					values = raidModeValues,
					name = LCo["Tooltips"],
					width = "double",
					order = 13,
				},
				achievementsTradeSkills = {
					type = "select",
					values = raidModeValues,
					name = LCo["Achievements trade skills"],
					width = "double",
					order = 14,
				},
				announce = {
					type = "select",
					values = raidModeValues,
					name = LCo["Announce"],
					width = "double",
					order = 15,
				},
			}
		},
		other = {
			type = "group",
			name = LCo["Other"],
			order = 3,
			args = {
				partyHeader = {
					name = LCo["Show all party quests you don't have under a specific header"],
					type = "toggle",
					set = optPHSetter,
					order = 1,
					width = "full",
				},
				colorizeMoney = {
					name = LCo["Color money"],
					type= "toggle",
					order = 2,
					width = "full",
				},
				moneySeparator = {
					name = LCo["Money separator"],
					type= "select",
					values = moneySeparators,
					order = 3,
					width = "full",
				},
				hideDefaultBlizTracker = {
					name = LCo["Hide default Blizzard quests and achievements tracker"],
					type= "toggle",
					set = optHideBTr,
					order = 4,
					width = "full",
				},
				macro = {
					name = LCo["Macro Generator"],
					type = "execute",
					func = specialItemMacro,
					order = 5,
				},
			},
		},
		weightedLevel = {
			type = "group",
			name = LCo["Weighted quest level computation"],
			set = optWQLSetter,
			order = 4,
			args = {
				eliteAddLevel = {
					name = LCo["Level to add for an elite quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 1,
				},
				groupAddLevel = {
					name = LCo["Level to add per missing player for a group quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 2,
				},
				dungeonAddLevel = {
					name = LCo["Level to add per missing player for a dungeon quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 3,
				},
				heroicAddLevel = {
					name = LCo["Level to add per missing player for an heroic quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 4,
				},
				raidAddLevel = {
					name = LCo["Level to add per missing player for a raid quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 5,
				},
			},
		},
		questsTag = {
			type = "group",
			name = LCo["Custom quests tag"],
			set = optTagSetter,
			order = 5,
			args = {
				level = {
					name = LCo["Quest level tag"],
					type = "select",
					values = questLevelTag,
					width = "full",
					order = 1,
				},
				elite = {
					name = LCo["Elite quest tag"],
					type = "input",
					order = 2,
				},
				group = {
					name = LCo["Group quest tag"],
					type = "input",
					order = 3,
				},
				pvp = {
					name = LCo["PVP quest tag"],
					type = "input",
					order = 4,
				},
				raid = {
					name = LCo["Raid quest tag"],
					type = "input",
					order = 5,
				},
				dungeon = {
					name = LCo["Dungeon quest tag"],
					type = "input",
					order = 6,
				},
				heroic = {
					name = LCo["Heroic quest tag"],
					type = "input",
					order = 7,
				},
				daily = {
					name = LCo["Daily quest tag"],
					type = "input",
					order = 8,
				},
				escort = {
					name = LCo["Escort quest tag"],
					type = "input",
					order = 9,
				},
			},
		},
		customColor = {
			type = "group",
			name = LCo["Custom objective coloring function"],
			set = optCCFSetter,
			order = 6,
			args = {
				easyColorSetupDesc = {
					type = "description",
					name = LCo["You can define a color for percentage of completion = 0, 50% and 100%. tomQuest2 will automatically define the needed color function between those threee points"],
					width = "full",
					order = 0,
				},
				zeroColor = {
					name = LCo["Color for completion = 0%"],
					type = "color",
					hasAlpha = false,
					get = zeroColorGetter,
					set = zeroColorSetter,
					width = "full",
					order = 1,
				},
				halfColor = {
					name = LCo["Color for completion = 50%"],
					type = "color",
					hasAlpha = false,
					get = halfColorGetter,
					set = halfColorSetter,
					width = "full",
					order = 2,
				},
				fullColor = {
					name = LCo["Color for completion = 100%"],
					type = "color",
					hasAlpha = false,
					get = fullColorGetter,
					set = fullColorSetter,
					width = "full",
					order = 3,
				},
				resetColor = {
					name = LCo["Reset color functions"],
					type = "execute",
					func = setDefaultColor,
					width = "full",
					order = 4,
				},
				customColorMode = {
					name = LCo["Hide custom color function setup"],
					type = "toggle",
					get = optGetter,
					set = optSetter,
					width = "full",
					order = 5,
				},
				customColorFunctionDesc = {
					type = "description",
					name = LCo["Custom objective coloring allows you to define multiplicative and additive component of the coloring function. For each color component you'll have a function that is color = (multiplicative component) * percent + (additive component) where percent is the percent of completion of the objective."],
					width = "full",
					hidden = hideCustomColorMode,
					order = 6,
				},
				zeroCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for objective completion at 0%"],
					width = "full",
					hidden = hideCustomColorMode,
					order = 7,
				},
				rj0 = {
					name = LCo["Red add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 8,
				},
				gj0 = {
					name = LCo["Green add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 9,
				},
				bj0 = {
					name = LCo["Blue add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 10
				},
				firstCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for [0%, 50%[ objective completion"],
					hidden = hideCustomColorMode,
					order = 11,
				},
				ri1 = {
					name = LCo["Red mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 12,
				},
				gi1 = {
					name = LCo["Green mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 13,
				},
				bi1 = {
					name = LCo["Blue mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 14,
				},
				rj1 = {
					name = LCo["Red add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 15,
				},
				gj1 = {
					name = LCo["Green add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 16,
				},
				bj1 = {
					name = LCo["Blue add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 17,
				},
				secondCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for ]50%, 100%[ objective completion"],
					hidden = hideCustomColorMode,
					order = 18,
				},
				ri2 = {
					name = LCo["Red mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 19,
				},
				gi2 = {
					name = LCo["Green mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 20,
				},
				bi2 = {
					name = LCo["Blue mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 21,
				},
				rj2 = {
					name = LCo["Red add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 22,
				},
				gj2 = {
					name = LCo["Green add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 23,
				},
				bj2 = {
					name = LCo["Blue add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 24,
				},
				fullCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for objective completion at 100%"],
					hidden = hideCustomColorMode,
					order = 25,
				},
				rj3 = {
					name = LCo["Red add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 26,
				},
				gj3 = {
					name = LCo["Green add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 27,
				},
				bj3 = {
					name = LCo["Blue add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 28,
				},
			},
		},
		questStatusColor = {
			type = "group",
			name = LCo["Quest status color"],
			get = hexColorGetter,
			set = hexColorSetter,
			order = 7,
			args = {
				completedColor = {
					name = LCo["Color for quest complete"],
					type = "color",
					hasAlpha = false,
					width = "full",
					order = 1,
				},
				failedColor = {
					name = LCo["Color for quest failed"],
					type = "color",
					hasAlpha = false,
					width = "full",
					order = 2,
				},
			},
		},
		sorting = {
			type = "group",
			name = LCo["Zones and quests sorting"],
			set = sortSetter,
			order = 8,
			args = {
				categorySorting = {
					type = "select",
					name = LCo["Zones sorting method"],
					values = zSorting,
					width = "full",
					order = 1,
				},
				categoryWithoutLvlPos = {
					type = "select",
					name = LCo["Position of zones without level when sorting zones by level"],
					set = zwlSetter,
					values = zwlPosition,
					width = "full",
					order = 2,
				},
				questsSorting = {
					type = "select",
					name = LCo["Quests sorting method"],
					values = qSorting,
					width = "full",
					order = 3,
				},
			},
		},
	}
}

if IsAddOnLoaded("ButtonFacade") then
	local function getLBF(info)
	return tomQuest2.db.profile.skin[info[#info]]
	end
	local function setSkin(info, value)
		tomQuest2.LBFGroup:Skin(value, tomQuest2.LBFGroup.Gloss, tomQuest2.LBFGroup.Backdrop)
	end
	local function setGloss(info, value)
		tomQuest2.LBFGroup:Skin(tomQuest2.LBFGroup.SkinID, value, tomQuest2.LBFGroup.Backdrop)
	end
	local function setBackdrop(info, value)
		tomQuest2.LBFGroup:Skin(tomQuest2.LBFGroup.SkinID, tomQuest2.LBFGroup.Gloss, value and true or false)
	end
	local function getLayerColor(info)
		return tomQuest2.LBFGroup:GetLayerColor(info[#info])
	end
	local function setLayerColor(info, r,g,b,a)
		tomQuest2.LBFGroup:Skin(tomQuest2.LBFGroup.SkinID, tomQuest2.LBFGroup.Gloss, tomQuest2.LBFGroup.Backdrop, info[#info], r, g, b, a)
	end
	local function resetColors(info)
		tomQuest2.LBFGroup:ResetColors()
	end
	local LBF = LibStub("LibButtonFacade", true)
	local LB = LibStub("AceLocale-3.0"):GetLocale("ButtonFacade")
	options.args.buttonFacade = {
		type = "group",
		name = "Button Facade",
		get = getLBF,
		order = 9,
		args = {
			skin = {
				type = "select",
				name = LB["Skin"],
				desc = LB["Set the skin."],
				set = setSkin,
				style = "dropdown",
				width = "full",
				values = LBF.ListSkins,
				order = 2,
			},
			gloss = {
				type = "range",
				name = LB["Gloss"],
				desc = LB["Set the intensity of the gloss."],
				set = setGloss,
				min = 0,
				max = 1,
				step = 0.01,
				width = "full",
				isPercent = true,
				order = 3,
			},
			backdrop = {
				type = "toggle",
				name = LB["Backdrop"],
				desc = LB["Toggle the backdrop."],
				set = setBackdrop,
				width = "half",
				order = 4,
			},
			colorHeader = {
				type = "header",
				name = LB["Color Options"],
				order = 5,
			},
			Backdrop = {
				type = "color",
				name = LB["Backdrop"],
				desc = LB["Set the %s color."]:format(LB["Backdrop"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 6,
			},
			Flash = {
				type = "color",
				name = LB["Flash"],
				desc = LB["Set the %s color."]:format(LB["Flash"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 7,
			},
			Normal = {
				type = "color",
				name = LB["Normal"],
				desc = LB["Set the %s color."]:format(LB["Normal"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 8,
			},
			Pushed = {
				type = "color",
				name = LB["Pushed"],
				desc = LB["Set the %s color."]:format(LB["Pushed"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 9,
			},
			Disabled = {
				type = "color",
				name = LB["Disabled"],
				desc = LB["Set the %s color."]:format(LB["Disabled"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 10,
			},
			Checked = {
				type = "color",
				name = LB["Checked"],
				desc = LB["Set the %s color."]:format(LB["Checked"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 11,
			},
			Border = {
				type = "color",
				name = LB["Equipped"],
				desc = LB["Set the %s color."]:format(LB["Equipped"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 12,
			},
			Highlight = {
				type = "color",
				name = LB["Highlight"],
				desc = LB["Set the %s color."]:format(LB["Highlight"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 13,
			},
			Gloss = {
				type = "color",
				name = LB["Gloss"],
				desc = LB["Set the %s color."]:format(LB["Flash"]),
				get = getLayerColor,
				set = setLayerColor,
				order = 14,
			},
			resetColors = {
				type = "execute",
				name = LB["Reset Colors"],
				desc = LB["Reset all colors."],
				func = resetColors,
				order = 15,
			},
		},
	}
end

-----------------------------------------
--- Achievements Tracker option table ---
-----------------------------------------
local achievementPosition = {[1] = LCo["Above"], [2] = LCo["Below"]}
local outlines = {[""] = LCo["None"], ["OUTLINE"] = LCo["Outline"], ["THICKOUTLINE"] = LCo["Thick outline"]}
local achievementInteractivityList = {
	["chatLink"] = LCo["Insert achievement link in chat"],
	["achievementTooltip"] = LCo["Display of achievement tooltip"],
	["removeTrackedAchievement"] = LCo["Remove achievement from tracker"],
	["displayAchievement"] = LCo["Diplay achievement frame"],
	["nothing"] = LCo["Do nothing"],
}
local function hideAchievementInteraction()
	if tomQuest2.db.profile.achievementsTracker.disableInteraction then
		return true
	else
		return false
	end
end
local function achievementOptGetter(info)
	return tomQuest2.db.profile.achievementsTracker[info[#info]]
end
local function achievementOptSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2:setAchievementsTrackerLayout()
end
local function timedAchievementSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	if not tomQuest2.moduleState.achievementsTracker then return end
	if value then
		tomQuest2:RegisterEvent("TRACKED_ACHIEVEMENT_UPDATE")
	else
		tomQuest2:UnregisterEvent("TRACKED_ACHIEVEMENT_UPDATE")
	end
end
local function achievementInQuestsTrackerSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	if value then
		tomQuest2.ACHIEVEMENTS_TRACKER_FRAME = "questsTracker"
		for _, frame in pairs(tomQuest2.achievementCategoryTextCells) do
			frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementCategoryStatusCells) do
			frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementCells) do
			frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementDescriptionCells) do
			frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementStatusBars) do
			frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
		end
		for aid in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementCriterionTextCells[aid] then
				for cid, frame in pairs(tomQuest2.achievementCriterionTextCells[aid]) do
					frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
				end
			end
			if tomQuest2.achievementCriterionStatusCells[aid] then
				for cid, frame in pairs(tomQuest2.achievementCriterionStatusCells[aid]) do
					frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
				end
			end
			if tomQuest2.achievementBooleanCriterionCells[aid] then
				for cid, frame in pairs(tomQuest2.achievementBooleanCriterionCells[aid]) do
					frame:SetParent(tomQuest2.displayFrames.questsTracker.scrollChild)
				end
			end
		end
		tomQuest2:setAchievementsTrackerLayout()
	else
		tomQuest2.ACHIEVEMENTS_TRACKER_FRAME = "achievementsTracker"
		for _, frame in pairs(tomQuest2.achievementCategoryTextCells) do
			frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementCategoryStatusCells) do
			frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementCells) do
			frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementDescriptionCells) do
			frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
		end
		for _, frame in pairs(tomQuest2.achievementStatusBars) do
			frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
		end
		for aid in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementCriterionTextCells[aid] then
				for cid, frame in pairs(tomQuest2.achievementCriterionTextCells[aid]) do
					frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
				end
			end
			if tomQuest2.achievementCriterionStatusCells[aid] then
				for cid, frame in pairs(tomQuest2.achievementCriterionStatusCells[aid]) do
					frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
				end
			end
			if tomQuest2.achievementBooleanCriterionCells[aid] then
				for cid, frame in pairs(tomQuest2.achievementBooleanCriterionCells[aid]) do
					frame:SetParent(tomQuest2.displayFrames.achievementsTracker.scrollChild)
				end
			end
		end
		tomQuest2:setQuestsTrackerLayout()
		tomQuest2:setAchievementsTrackerLayout()
	end
end
local function lockAchievementTrackerSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.displayFrames["achievementsTracker"]:SetMovable(not value)
end
local function achievementCatFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.cATrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.categoryFont), tomQuest2.db.profile.achievementsTracker.categoryFontSize, tomQuest2.db.profile.achievementsTracker.categoryFontOutline)
	for cid in pairs(tomQuest2.achievementsCategories) do
		tomQuest2.achievementCategoryTextCells[cid].fs:SetFontObject(tomQuest2.cATrFontTQ2)
		tomQuest2.achievementCategoryStatusCells[cid].fs:SetFontObject(tomQuest2.cATrFontTQ2)
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementSubCatFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.scATrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.subCategoryFont), tomQuest2.db.profile.achievementsTracker.subCategoryFontSize, tomQuest2.db.profile.achievementsTracker.subCategoryFontOutline)
	for cid, cData in pairs(tomQuest2.achievementsCategories) do
		if next(cData.subCategory) then
			for scid, scData in pairs(cData.subCategory) do
				tomQuest2.achievementCategoryTextCells[scid].fs:SetFontObject(tomQuest2.scATrFontTQ2)
				tomQuest2.achievementCategoryStatusCells[scid].fs:SetFontObject(tomQuest2.scATrFontTQ2)
			end
		end
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.aATrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.achievementsTracker.achievementFont), tomQuest2.db.profile.achievementsTracker.achievementsTracker.achievementFontSize, tomQuest2.db.profile.achievementsTracker.achievementsTracker.profile.achievementFontOutline)
	for aid in pairs(tomQuest2.achievements) do
		tomQuest2.achievementCells[aid].fs:SetFontObject(tomQuest2.aATrFontTQ2)
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementCritFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.acATrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.achievementsTracker.achievementCriterionFont), tomQuest2.db.profile.achievementsTracker.achievementsTracker.achievementCriterionFontSize, tomQuest2.db.profile.achievementsTracker.achievementsTracker.achievementCriterionFontOutline)
	for aid, aData in pairs(tomQuest2.achievements) do
		if tomQuest2.achievementDescriptionCells[aid] then
			tomQuest2.achievementDescriptionCells[aid].fs:SetFontObject(tomQuest2.acATrFontTQ2)
		end
		if aData.criteria then
			for cid in pairs(aData.criteria) do
				tomQuest2.achievementCriterionTextCells[aid][cid].fs:SetFontObject(tomQuest2.acATrFontTQ2)
				tomQuest2.achievementCriterionStatusCells[aid][cid].fs:SetFontObject(tomQuest2.acATrFontTQ2)
			end
		end
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementStatusBarFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.sbATrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.achievementsTracker.statusBarFont), tomQuest2.db.profile.achievementsTracker.achievementsTracker.statusBarFontSize, tomQuest2.db.profile.achievementsTracker.achievementsTracker.statusBarFontOutline)
	for aid in pairs(tomQuest2.achievements) do
		if tomQuest2.achievementStatusBars[aid] then
			tomQuest2.achievementStatusBars[aid].fs:SetFontObject(tomQuest2.sbATrFontTQ2)
		end
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementsTrackerBackdropSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2:setBackDrop("achievementsTracker")
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementsTrackerCategoryCollapseButtonSizeSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	for cid in pairs(tomQuest2.achievementsCategories) do
		tomQuest2.categoryButtons.achievementsTracker[cid]:SetScale(value/tomQuest2.categoryButtons.achievementsTracker[cid].height)
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementsTrackerSubCategoryCollapseButtonSizeSetter(info, value)
	for cid, cData in pairs(tomQuest2.achievementsCategories) do
		if next(cData.subCategory) then
			for scid, scData in pairs(cData.subCategory) do
				tomQuest2.categoryButtons.achievementsTracker[scid]:SetScale(value/tomQuest2.categoryButtons.achievementsTracker[scid].height)
			end
		end
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function achievementColorGetter(info)
	return unpack(tomQuest2.db.profile.achievementsTracker[info[#info]])
end
local function achievementColorSetter(info, r, g, b, a)
	tomQuest2.db.profile.achievementsTracker[info[#info]][1] = r
	tomQuest2.db.profile.achievementsTracker[info[#info]][2] = g
	tomQuest2.db.profile.achievementsTracker[info[#info]][3] = b
	tomQuest2.db.profile.achievementsTracker[info[#info]][4] = a
	if info[#info] == "defaultStatusBarColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementStatusBars[aid] then
				local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, 1)
				local color = tomQuest2:getColorFromCompletionHEX(quantity / reqQuantity)
				tomQuest2:updateStatusbar(aid, cType, quantity, reqQuantity, color)
			end
		end
	end
	tomQuest2:setAchievementsTrackerLayout()
end
local function setAchievementTrackerColors(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	if info[#info] == "colorizeACName" or info[#info] == "colorizeACCounter" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			local numTA, numTCA = GetCategoryNumAchievements(cid)
			local color = tomQuest2:getColorFromCompletionHEX(numTCA / numTA)
			tomQuest2:updateAchievementCategoryCells(cid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, cData.categoryName, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
		end
	elseif info[#info] == "colorizeASCName" or info[#info] == "colorizeASCCounter" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			if next(cData.subCategory) then
				for scid, scData in pairs(cData.subCategory) do
					local numTA, numTCA = GetCategoryNumAchievements(cid)
					local color = tomQuest2:getColorFromCompletionHEX(numTCA / numTA)
					tomQuest2:updateAchievementCategoryCells(scid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, scData.categoryName, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
				end
			end
		end
	elseif "colorizeACrName" or info[#info] == "colorizeACrStatus" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if aData.criteria then
				for cid in pairs(aData.criteria) do
					local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, cid)
					local color = tomQuest2:getColorFromCompletionHEX(quantity / reqQuantity)
					tomQuest2.achievementCriterionTextCells[aid][cid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name)
					tomQuest2.achievementCriterionStatusCells[aid][cid].fs:SetFormattedText("|cff%s / %s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrStatus and color or tomQuest2.db.profile.achievementsTracker.defaultACrStatusColor, quantity, reqQuantity)
				end
			end
		end
	elseif info[#info] == "colorizeStatusBar" or info[#info] == "colorizeStatusBarText" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementStatusBars[aid] then
				local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, 1)
				local color = tomQuest2:getColorFromCompletionHEX(quantity / reqQuantity)
				tomQuest2:updateStatusbar(aid, cType, quantity, reqQuantity, color)
			end
		end
	end
end
local function achievementHexColorGetter(info)
	return tonumber(sub(tomQuest2.db.profile.achievementsTracker[info[#info]], 1, 2), 16) / 255, tonumber(sub(tomQuest2.db.profile.achievementsTracker[info[#info]], 3, 4), 16) / 255, tonumber(sub(tomQuest2.db.profile.achievementsTracker[info[#info]], 5, 6), 16) / 255
end
local function achievementHexColorSetter(info,r, g, b)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = format("%02x%02x%02x", r*255, g*255, b*255)
	if info[#info] == "defaultACNameColor" or info[#info] == "defaultACCounterColor" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			local numTA, numTCA = GetCategoryNumAchievements(cid)
			local color = tomQuest2:getColorFromCompletionHEX(numTCA / numTA)
			tomQuest2:updateAchievementCategoryCells(cid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, cData.categoryName, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
		end
	elseif info[#info] == "defaultASCNameColor" or info[#info] == "defaultASCCounterColor" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			if next(cData.subCategory) then
				for scid, scData in pairs(cData.subCategory) do
					local numTA, numTCA = GetCategoryNumAchievements(cid)
					local color = tomQuest2:getColorFromCompletionHEX(numTCA / numTA)
					tomQuest2:updateAchievementCategoryCells(scid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, scData.categoryName, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
				end
			end
		end
	elseif info[#info] == "defaultANameColor" then
		for aid in pairs(tomQuest2.achievements) do
			tomQuest2.achievementCells[aid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.defaultANameColor, tomQuest2.achievements[aid].name)
		end
	elseif info[#info] == "defaultACrNameColor" or info[#info] == "defaultACrStatusColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if aData.criteria then
				for cid in pairs(aData.criteria) do
					local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, cid)
					local color = tomQuest2:getColorFromCompletionHEX(quantity / reqQuantity)
					tomQuest2.achievementCriterionTextCells[aid][cid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name)
					tomQuest2.achievementCriterionStatusCells[aid][cid].fs:SetFormattedText("|cff%s / %s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrStatus and color or tomQuest2.db.profile.achievementsTracker.defaultACrStatusColor, quantity, reqQuantity)
				end
			end
		end
	elseif info[#info] == "defaultStatusBarTextColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementStatusBars[aid] then
				local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, 1)
				local color = tomQuest2:getColorFromCompletionHEX(quantity / reqQuantity)
				tomQuest2:updateStatusbar(aid, cType, quantity, reqQuantity, color)
			end
		end
	elseif info[#info] == "defaultDescriptionColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementDescriptionCells[aid] then
				tomQuest2.achievementDescriptionCells[aid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.defaultDescriptionColor, tomQuest2.achievements[aid].description)
			end
		end
	end
	tomQuest2:setAchievementsTrackerLayout()
end

local achievementsTrackerOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["Achievements tracker"],
	get = achievementOptGetter,
	set = achievementOptSetter,
	args = {
		autotrack = {
			type = "group",
			name = LCo["Auto track"],
			args = {
				timedAchievements = {
					name = LCo["Timed achievements"],
					type = "toggle",
					get = optGetter,
					set = timedAchievementSetter,
					width = "full",
					order = 1,
				},
			},
		},
		display = {
			type = "group",
			name = LCo["Display"],
			args = {
				showInQuestTracker = {
					name = LCo["Show achievements in quests tracker"],
					type = "toggle",
					set = achievementInQuestsTrackerSetter,
					width = "full",
					order = 1,
				},
				positionInQuestTracker = {
					name = LCo["Position of achievements relative to quests in the tracker"],
					type = "select",
					values = achievementPosition,
					set = achievementInQuestsTrackerSetter,
					width = "full",
					order = 2,
				},
				lockTracker = {
					name = LCo["Lock - unlock achievements tracker"],
					type = "toggle",
					set = lockAchievementTrackerSetter,
					width = "full",
					order = 4,
				},
				toggleTracker = {
					name = LCo["Display tracker"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				hideTrackerInCombat = {
					name = LCo["Hide achievements tracker in combat"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				hideCompletedCriteria = {
					name = LCo["Hide completed achievement criteria"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				showByCategory = {
					name = LCo["Display achievements by category and subcategory"],
					type = "toggle",
					width = "full",
					order = 8,
				},
			},
		},
		fonts = {
			type = "group",
			name = LCo["Tracker fonts"],
			args = {
				category = {
					type = "group",
					name = LCo["Achievement category"],
					set = achievementCatFontSetter,
					inline = true,
					order = 1,
					args = {
						categoryFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						categoryFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						categoryFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				subCategory = {
					type = "group",
					name = LCo["Achievement sub category"],
					set = achievementSubCatFontSetter,
					inline = true,
					order = 2,
					args = {
						subCategoryFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						subCategoryFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						subCategoryFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				achievement = {
					type = "group",
					name = LCo["Achievement"],
					set = achievementFontSetter,
					inline = true,
					order = 3,
					args = {
						achievementFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						achievementFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						achievementFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				achievementCriteria = {
					type = "group",
					name = LCo["Achievement criteria"],
					set = achievementCritFontSetter,
					inline = true,
					order = 4,
					args = {
						achievementCriterionFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						achievementCriterionFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						achievementCriterionFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				statusBar = {
					type = "group",
					name = LCo["Status bar"],
					set = achievementStatusBarFontSetter,
					inline = true,
					order = 5,
					args = {
						statusBarFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						statusBarFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						statusBarFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
			},
		},
		appearance = {
			type = "group",
			name = LCo["Tracker appearance"],
			args = {
				bgTexture = {
					name = LCo["Background texture"],
					type = "select",
					dialogControl = "LSM30_Background",
					values = AceGUIWidgetLSMlists.background,
					set = achievementsTrackerBackdropSetter,
					order = 1,
				},
				borderTexture = {
					name = LCo["Border texture"],
					type = "select",
					dialogControl = "LSM30_Border",
					values = AceGUIWidgetLSMlists.border,
					set = achievementsTrackerBackdropSetter,
					order = 2,
				},
				borderWidth = {
					name = LCo["Border width"],
					type = "range",
					min = 1,
					max = 24,
					step = 1,
					set = achievementsTrackerBackdropSetter,
					order = 3,
				},
				borderInset = {
					name = LCo["Inset"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					set = achievementsTrackerBackdropSetter,
					order = 4,
				},
				scale = {
					name = LCo["Achievements tracker scale"],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order = 5,
				},
				hasMaxHeight = {
					name = LCo["Set achievements tracker maximum height after which it will be scrollable"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				maxHeight = {
					name = LCo["Achievements tracker max height"],
					type = "range",
					min = 100,
					max = 2000,
					step = 5,
					width = "full",
					order = 7,
				},
				showCollapseButton = {
					name = LCo["Show collapse button"],
					type = "toggle",
					width = "full",
					order = 8,
				},
				categoryCollapseButtonSize = {
					name = LCo["Category collapse button size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					set = achievementsTrackerCategoryCollapseButtonSizeSetter,
					width = "full",
					order = 9,
				},
				subCategoryCollapseButtonSize = {
					name = LCo["Sub category collapse button size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					set = achievementsTrackerSubCategoryCollapseButtonSizeSetter,
					width = "full",
					order = 10,
				},
				showAchievementIcon = {
					name = LCo["Display achievements icon"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				achievementIconSize = {
					name = LCo["Achievement image size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 12,
				},
				subCategoryCellPadding = {
					name = LCo["Sub category cell padding"],
					type = "range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 13,
				},
				achievementCellPadding = {
					name = LCo["Achievement cell padding"],
					type = "range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 14,
				},
				criteriaCellPadding = {
					name = LCo["Achievement criteria cell padding"],
					type = "range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 15,
				},
				maxWidth = {
					name = LCo["Achievements tracker column max width"],
					type = "range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 17,
				},
				minWidth = {
					name = LCo["Achievements tracker column min width"],
					type = "range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 18,
				},
				statusMaxWidth = {
					name = LCo["Achievements tracker status column max width"],
					type = "range",
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 19,
				},
				statusMinWidth = {
					name = LCo["Achievements tracker status column min width"],
					type = "range",
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 20,
				},
				numBooleanCriteriaColumns = {
					name = LCo["Number of columns in the achievements tracker for boolean criteria"],
					type = "range",
					min = 1,
					max = 8,
					step = 1,
					width = "full",
					order = 21,
				},
			},
		},
		color = {
			type = "group",
			name = LCo["Colors"],
			args = {
				backDropColor = {
					name = LCo["Achievements tracker background color"],
					type = "color",
					hasAlpha = true,
					get = achievementColorGetter,
					set = achievementColorSetter,
					width = "full",
					order = 1,
				},
				borderColor = {
					name = LCo["Achievements tracker border color"],
					type = "color",
					hasAlpha = true,
					get = achievementColorGetter,
					set = achievementColorSetter,
					width = "full",
					order = 2,
				},
				colorizeACName = {
					name = LCo["Color achievements category name"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 3,
				},
				defaultACNameColor = {
					name = LCo["Default achievements category name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 4,
				},
				colorizeACCounter = {
					name = LCo["Color achievements category counter"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 5,
				},
				defaultACCounterColor = {
					name = LCo["Default achievements category counter color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 6,
				},
				colorizeASCName = {
					name = LCo["Color achievements subcategory name"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 7,
				},
				defaultASCNameColor = {
					name = LCo["Default achivements subcategory name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 8,
				},
				colorizeASCCounter = {
					name = LCo["Color achievements subcategory counter"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 9,
				},
				defaultASCCounterColor = {
					name = LCo["Default achievements subcategory counter color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 10,
				},
				defaultANameColor = {
					name = LCo["Default achievements name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 12,
				},
				colorizeATimer = {
					name = LCo["Color achievement timer"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 13,
				},
				defaultATimerColor = {
					name = LCo["Default achievement timer color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 14,
				},
				colorizeACrName = {
					name = LCo["Color achievements criteria name"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 15,
				},
				defaultACrNameColor = {
					name = LCo["Default achievements criteria name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 16,
				},
				colorizeACrStatus = {
					name = LCo["Color achievements cirteria status"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 17,
				},
				defaultACrStatusColor = {
					name = LCo["Default achievements criteria status color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 18,
				},
				colorizeStatusBar = {
					name = LCo["Color status bar"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 19,
				},
				defaultStatusBarColor = {
					name = LCo["Default status bar color"],
					type = "color",
					hasAlpha = false,
					get = achievementColorGetter,
					set = achievementColorSetter,
					width = "full",
					order = 20,
				},
				colorizeStatusBarText ={
					name = LCo["Color status bar text"],
					type = "toggle",
					set = setAchievementTrackerColors,
					width = "full",
					order = 21,
				},
				defaultStatusBarTextColor = {
					name = LCo["Default status bar text color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 22,
				},
				defaultDescriptionColor = {
					name = LCo["Default achievement description color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 23,
				},
			},
		},
		statusBar = {
			type = "group",
			name = LCo["Status bar"],
			args = {
				displayAsStatusBar = {
					name = LCo["Display status bar for criteria progression when possible"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				statusBarTexture = {
					name = LCo["Status bar texture"],
					type = "select",
					dialogControl = 'LSM30_Statusbar',
					values = AceGUIWidgetLSMlists.statusbar,
					order = 2,
				},
				statusBarHeight = {
					name = LCo["Status bar height"],
					type = "range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 3,
				},
				leftStatusBarPadding = {
					name = LCo["Left padding of the status bar"],
					type = "range",
					min = 0,
					max = 100,
					step = 1,
					width = "full",
					order = 4,
				},
				rightStatusBarPadding = {
					name = LCo["Right padding of the status bar"],
					type = "range",
					min = 0,
					max = 100,
					step = 1,
					width = "full",
					order = 5,
				},
			},
		},
		interaction = {
			type = "group",
			name = LCo["Tracker interaction"],
			args = {
				disableInteraction = {
					type = "toggle",
					name = LCo["Completely disable tracker interaction, should not be used when display by category is used or if you wish to make it scrollable."],
					width = "full",
					order = 1,
				},
				overTooltip = {
					name = LCo["Display achievement tooltip on mouse over"],
					type = "toggle",
					width = "full",
					hidden = hideAchievementInteraction,
					order = 2,
				},
				mouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 3,
				},
				mouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 4,
				},
				mouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 5,
				},
				mouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 6,
				},
				mouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 7,
				},
				mouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 8,
				},
				mouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 9,
				},
				mouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 10,
				},
				mouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 11,
				},
				mouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 12,
				},
				mouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 13,
				},
				mouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = hideAchievementInteraction,
					order = 14,
				},
			},
		},
	},
}

-----------------------------
--- Announce option table ---
-----------------------------
local announceOptions = {
	type = "group",
	name = LCo["Announce"],
	get = optGetter,
	set = optSetter,
	args = {
		announcePlayerObjectiveProgress = {
			name = LCo["Quests objectives progress annoucement"],
			type = "toggle",
			width = "full",
			order = 1,
		},
		announcePlayerObjectiveComplete = {
			name = LCo["Quests objectives complete announcement"],
			type = "toggle",
			width = "full",
			order = 2,
		},
		announcePlayerQuestComplete = {
			name = LCo["Quests complete announcement"],
			type = "toggle",
			width = "full",
			order = 3,
		},
		announcePlayerQuestFailed = {
			name = LCo["Quests failed announcement"],
			type = "toggle",
			width = "full",
			order = 4,
		},
		announceObjectiveProgressWhenQuestComplete = {
			name = LCo["Objective announce on quest completion"],
			type = "toggle",
			width = "full",
			order = 5,
		}
	},
}

----------------------------------
--- Daily tooltip option table ---
----------------------------------
local ldbInteractivityList = {
	["openOptions"] = LCo["Open tomQuest2 option frame"],
	["toggleQuestTracker"] = LCo["Toggle quests tracking frame"],
	["toggleAchievementsTracker"] = LCo["Toggle achievements tracking frame"],
	["toggleAnnouncementModule"] = LCo["toggle chat announcement module"],
	["toggleQuestsLog"] = LCo["Toggle Blizzard quests log"],
	["toggleSyncPartyModule"] = LCo["Toggle party synchronization module"],
	["toggleQuestsTrackerModule"] = LCo["Toggle quests tracking module"],
	["toggleAchievementsTrackerModule"] = LCo["Toggle achievements tracking module"],
	["lockQuestsTracker"] = LCo["Lock - unlock quests tracker"],
	["lockAchievementsTracker"] = LCo["Lock - unlock achievements tracker"],
	["nothing"] = LCo["Do nothing"],
}
local function dailyTooltipDailySetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	tomQuest2:updateDQToText()
end
local function dailyTooltipTimeSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	if value then
		tomQuest2.dailyTimer = tomQuest2:ScheduleRepeatingTimer("updateDQToText", 60)
	else
		tomQuest2:CancelTimer(tomQuest2.dailyTimer, true)
		tomQuest2.dailyTimer = nil
	end
	tomQuest2:updateDQToText()
end

local dailyTooltipOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["Daily tooltip"],
	get = optGetter,
	set = optSetter,
	args = {
		counter = {
			type = "group",
			name = LCo["Daily quests counter"],
			set = dailyTooltipDailySetter,
			args = {
				showNumCompletedDailyQuests = {
					name = LCo["Show number of completed daily quest"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				showMaxDailyQuest = {
					name = LCo["Show maximum daily quests number"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				showRemainingTime = {
					name = LCo["Show remaining time before end of day"],
					type = "toggle",
					set = dailyTooltipTimeSetter,
					width = "full",
					order = 3,
				},
				colorizeDailyQuestsCounter = {
					name = LCo["Color daily quests counter"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				colorizeDailyQuestsTimer = {
					name = LCo["Color daily quests timer"],
					type = "toggle",
					width = "full",
					order = 5,
				},
			},
		},
		tooltip = {
			type = "group",
			name = LCo["Daily tooltip"],
			args = {
				showCOTD = {
					name = LCo["Show in the tooltip daily quests completed today"],
					type = "toggle",
					width = "full",
					order = 1,
				},
			},
		},
		ldbInteraction = {
			type = "group",
			name = LCo["Daily quests counter interaction"],
			args = {
				dqToMouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 1,
				},
				dqToMouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 2,
				},
				dqToMouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 3,
				},
				dqToMouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 4,
				},
				dqToMouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 5,
				},
				dqToMouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 6,
				},
				dqToMouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 7,
				},
				dqToMouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 8,
				},
				dqToMouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 9,
				},
				dqToMouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 10,
				},
				dqToMouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 11,
				},
				dqToMouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 12,
				},
			},
		},
	},
}

----------------------------------
--- Notifications option table ---
----------------------------------
local function hideBlizzardNotificationSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	tomQuest2:setBlizzNotification()
end

local notificationsOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["Notifications"],
	get = optGetter,
	set = optSetter,
	args = {
		common = {
			type = "group",
			name = LCo["Common settings"],
			args = {
				hideBlizzardNotification = {
					name = LCo["Hide default Blizzard quests notification"],
					type = "toggle",
					set = hideBlizzardNotificationSetter,
					width = "full",
					order = 1,
				},
				notifiyObjectiveProgressWhenQuestComplete = {
					name = LCo["Objective notification on quest completion"],
					type = "toggle",
					width = "full",
					order = 2,
				},
			},
		},
		player = {
			type = "group",
			name = LCo["Player notifications"],
			args = {
				notifyPlayerObjectiveProgress = {
					name = LCo["Quest objective progress notification"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				notifyPlayerObjectiveComplete = {
					name = LCo["Quest objective complete notification"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				notifyPlayerQuestComplete = {
					name = LCo["Quest complete notification"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				notifyPlayerQuestFailed = {
					name = LCo["Quest failed notification"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				notifyItemStartingQuestLooted = {
					name = LCo["Item starting a quest notification"],
					type = "toggle",
					width = "full",
					order = 5,
				},
			},
		},
		party = {
			type = "group",
			name = LCo["Party notifications"],
			args = {
				notifyPartyObjectiveProgress = {
					name = LCo["Quest objective progress notification"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				notifyPartyObjectiveComplete = {
					name = LCo["Quest objective complete notification"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				notifyPartyQuestComplete = {
					name = LCo["Quest complete notification"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				notifyPartyQuestFailed = {
					name = LCo["Quest failed notification"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				notifyPartyQuestAccepted = {
					name = LCo["Quest accepted notification"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				notifyPartyQuestTurnedIn = {
					name = LCo["Quest turned in notification"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				notifyPartyQuestAbandonned = {
					name = LCo["Quest abandonned notification"],
					type = "toggle",
					width = "full",
					order = 7,
				},
			},
		},
		output = tomQuest2:GetSinkAce3OptionsDataTable()
	},
}

----------------------------------
--- Quests Givers option table ---
----------------------------------
local automaticAcceptChoices = {[1] = LCo["All quests"], [2] = LCo["Only daily quests"], [3] = LCo["All but daily quests"], [4] = LCo["Never"]}

local function questsGiversHexColorSetter(info, arg1, arg2, arg3)
	tomQuest2.db.profile[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
end

local questsGiversOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["Quests givers"],
	get = optGetter,
	set = optSetter,
	args = {
		questTitleColor = {
			name = LCo["Quest title color"],
			type = "color",
			hasAlpha = false,
			set = questsGiversHexColorSetter,
			width = "full",
			order = 1,
		},
		addQuestLevel = {
			name = LCo["Add quest level to the quest name"],
			type = "toggle",
			width = "full",
			order = 2,
		},
		fadeNotCompletedQuests = {
			name = LCo["Fade quest name that are not complete"],
			type = "toggle",
			width = "full",
			order = 3,
		},
		automaticAccept = {
			name = LCo["Accept quests automatically"],
			type = "select",
			values = automaticAcceptChoices,
			width = "full",
			order = 4,
		},
		automaticAcceptEscort = {
			name = LCo["Accept escort quest started by other player automatically"],
			type = "toggle",
			width = "full",
			order = 5,
		},
		automaticTurnIn = {
			name = LCo["Automatic quest turn-in"],
			type = "toggle",
			width = "full",
			order = 6,
		},
	},
}

--------------------------------------
--- Quets Information option table ---
--------------------------------------
local attachSelect = {[1] = LCo["Right"], [2] = LCo["Left"]}

local function questsInformationsColorGetter(info)
	return unpack(tomQuest2.db.profile[info[#info]])
end
local function questsInformationsColorSetter(info, arg1, arg2, arg3, arg4)
	tomQuest2.db.profile[info[#info]][1] = arg1
	tomQuest2.db.profile[info[#info]][2] = arg2
	tomQuest2.db.profile[info[#info]][3] = arg3
	tomQuest2.db.profile[info[#info]][4] = arg4
	local qid
	if tomQuest2ParentFrame then
		tomQuest2ParentFrame:SetBackdropColor(tomQuest2.db.profile.backDropColor[1], tomQuest2.db.profile.backDropColor[2], tomQuest2.db.profile.backDropColor[3], tomQuest2.db.profile.backDropColor[4])
		tomQuest2ParentFrame:SetBackdropBorderColor(tomQuest2.db.profile.borderColor[1], tomQuest2.db.profile.borderColor[2], tomQuest2.db.profile.borderColor[3], tomQuest2.db.profile.borderColor[4])
	end
	if tomQuest2QlFrame then
		tomQuest2QlFrame:SetBackdropColor(tomQuest2.db.profile.backDropColor[1], tomQuest2.db.profile.backDropColor[2], tomQuest2.db.profile.backDropColor[3], tomQuest2.db.profile.backDropColor[4])
		tomQuest2QlFrame:SetBackdropBorderColor(tomQuest2.db.profile.borderColor[1], tomQuest2.db.profile.borderColor[2], tomQuest2.db.profile.borderColor[3], tomQuest2.db.profile.borderColor[4])
		tomQuest2:lockUnlockQlFrame()
		qid = tomQuest2QlFrame.currentQid
		tomQuest2QlFrame.currentQid = nil
	end
	if tomQuest2LhFrame then
		tomQuest2LhFrame:SetBackdropColor(tomQuest2.db.profile.backDropColor[1], tomQuest2.db.profile.backDropColor[2], tomQuest2.db.profile.backDropColor[3], tomQuest2.db.profile.backDropColor[4])
		tomQuest2LhFrame:SetBackdropBorderColor(tomQuest2.db.profile.borderColor[1], tomQuest2.db.profile.borderColor[2], tomQuest2.db.profile.borderColor[3], tomQuest2.db.profile.borderColor[4])
		tomQuest2:lockUnlockLhFrame()
		qid = tomQuest2LhFrame.currentQid or qid
		tomQuest2LhFrame.currentQid = nil
	end
	if qid then
		tomQuest2:displayQuestInformations(nil, qid)
	end
end
local function questsInformationsFrameSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	local qid
	if tomQuest2QlFrame then
		tomQuest2:lockUnlockQlFrame()
		qid = tomQuest2QlFrame.currentqid
		tomQuest2QlFrame.currentQid = nil
	end
	if tomQuest2LhFrame then
		tomQuest2:lockUnlockLhFrame()
		qid = tomQuest2LhFrame.currentqid or qid
		tomQuest2LhFrame.currentQid = nil
	end
	if qid then
		tomQuest2:displayQuestInformations(nil, qid)
	end
end

local questsInformationsOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["Information"],
	get = optGetter,
	set = optSetter,
	args = {
		common = {
			type = "group",
			name = LCo["Common settings"],
			args = {
				backDropColor = {
					name = LCo["Background color"],
					type = "color",
					hasAlpha = true,
					get = questsInformationsColorGetter,
					set = questsInformationsColorSetter,
					width = "full",
					order = 1,
				},
				borderColor = {
					name = LCo["Border color"],
					type = "color",
					hasAlpha = true,
					get = questsInformationsColorGetter,
					set = questsInformationsColorSetter,
					width = "full",
					order = 2,
				},
				attach = {
					name = LCo["Attach LightHeaded to quests log"],
					type = "select",
					values = attachSelect,
					set = questsInformationsFrameSetter,
					width = "full",
					order = 3,
				},
			},
		},
		questLog = {
			type = "group",
			name = LCo["Quests log settings"],
			args = {
				showQuestLog = {
					name = LCo["Show tomQuest2 quest log"],
					type = "toggle",
					set = questsInformationsFrameSetter,
					width = "full",
					order = 1,
				},
				qlFont = {
					name = LCo["Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					width = "full",
					order = 2,
				},
				qlFontSizeH1 = {
					name = LCo["Quest title font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 3,
				},
				qlFontSizeH2 = {
					name = LCo["Headers font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 4,
				},
				qlFontSize = {
					name = LCo["Text font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 5,
				},
			},
		},
		lightHeaded = {
			type = "group",
			name = LCo["LightHeaded settings"],
			args = {
				showLightHeaded = {
					name = LCo["Show LightHeaded into tomQuest2"],
					type = "toggle",
					set = questsInformationsFrameSetter,
					width = "full",
					order = 1,
				},
				showAllCommentsOnOnePage = {
					name = LCo["Show all LightHeaded comments on the same page"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				lhFont = {
					name = LCo["Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					width = "full",
					order = 3,
				},
				lhFontSize = {
					name = LCo["Font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 4,
				},
			},
		},
	},
}

-----------------------------------
--- Quests Tooltip option table ---
-----------------------------------
local hideObjectives = {[1] = LCo["When you've completed the objective"], [2] = LCo["When everyone in the party has completed the objective"], [3] = LCo["Never"]}
local function qTOOptGetter(info)
	return tomQuest2.db.profile.questsTooltip[info[#info]]
end
local function qTOOptSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
end
local function questsTooltipLdbTextSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	tomQuest2:updateQuestCounter()
end
local function hZLOptSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	for category in pairs(tomQuest2.questsCategories) do
		tomQuest2:updateQuestCategoryCell(category, "questsTooltip")
	end
end
local function hQTOptSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	for qid in pairs(tomQuest2.quests) do
		tomQuest2:updateQuestTextCell(qid, "questsTooltip")
	end
end
local function questsTooltipCategoryFontSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	tomQuest2.cQToFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.categoryFont), tomQuest2.db.profile.questsTooltip.categoryFontSize, tomQuest2.db.profile.questsTooltip.categoryFontOutline)
	for category in pairs(tomQuest2.questsCategories) do
		tomQuest2.questCategoryCells.questsTooltip[category].fs:SetFontObject(tomQuest2.cQToFontTQ2)
	end
end
local function questsTooltipQuestFontSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	tomQuest2.qQToFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.questFont), tomQuest2.db.profile.questsTooltip.questFontSize, tomQuest2.db.profile.questsTooltip.questFontOutline)
	for qid in pairs(tomQuest2.quests) do
		tomQuest2.questTextCells.questsTooltip[qid].fs:SetFontObject(tomQuest2.qQToFontTQ2)
		tomQuest2.questStatusCells.questsTooltip[qid].fs:SetFontObject(tomQuest2.qQToFontTQ2)
	end
end
local function questsTooltipQuestObjectiveFontSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	tomQuest2.qoQToFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.questObjectiveFont), tomQuest2.db.profile.questsTooltip.questObjectiveFontSize, tomQuest2.db.profile.questsTooltip.questObjectiveFontOutline)
	for qid, qoData in pairs(tomQuest2.questsObjectives) do
		for oid in pairs(qoData) do
			tomQuest2.questsObjectiveTextCells.questsTooltip[qid][oid].fs:SetFontObject(tomQuest2.qoQToFontTQ2)
			tomQuest2.questsObjectiveStatusCells.questsTooltip[qid][oid].fs:SetFontObject(tomQuest2.qoQToFontTQ2)
		end
	end
end
local function questsTooltipBackdropSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
		tomQuest2:setBackDrop("questsTooltip")
end
local function questsTooltipCollapseButtonSizeSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	for _, button in pairs(tomQuest2.categoryButtons.questsTooltip) do
		button:SetScale(value/button.height)
	end
end
local function questsTooltipColorGetter(info)
	return unpack(tomQuest2.db.profile.questsTooltip[info[#info]])
end
local function questsTooltipColorSetter(info, r, g, b, a)
	tomQuest2.db.profile.questsTooltip[info[#info]][1] = r
	tomQuest2.db.profile.questsTooltip[info[#info]][2] = g
	tomQuest2.db.profile.questsTooltip[info[#info]][3] = b
	tomQuest2.db.profile.questsTooltip[info[#info]][4] = a
end
local function setQuestsTooltipColors(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	if info[#info] == "colorizeCategoryName" or info[#info] == "colorizeCategoryLevel" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2:updateQuestCategoryCell(category, "questsTooltip")
		end
	elseif info[#info] == "colorizeQuestName" or info[#info] == "colorizeQuestLevel" then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2:updateQuestTextCell(qid, "questsTooltip")
		end
	elseif info[#info] == "colorizeQuestObjective" or info[#info] == "colorizeQuestObjectiveCounter" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2:updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTooltip[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTooltip.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTooltip[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTooltip.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor)
			end
		end
	end
end
local function questsTooltipHexColorGetter(info)
	return tonumber(sub(tomQuest2.db.profile.questsTooltip[info[#info]], 1, 2), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTooltip[info[#info]], 3, 4), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTooltip[info[#info]], 5, 6), 16) / 255
end
local function questsTooltipHexColorSetter(info, arg1, arg2, arg3)
	tomQuest2.db.profile.questsTooltip[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
	if info[#info] == "defaultCategoryNameColor" or info[#info] == "defaultCategoryLevelColor" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2:updateQuestCategoryCell(category, "questsTooltip")
		end
	elseif info[#info] == "defaultQuestNameColor" or info[#info] == "defaultQuestLevelColor" then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2:updateQuestTextCell(qid, "questsTooltip")
		end
	elseif info[#info] == "defaultQuestObjectiveColor" or info[#info] == "defaultQuestObjectiveCounterColor" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2:updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTooltip[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTooltip.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTooltip[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTooltip.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor)
			end
		end
	end
end
local function questsTooltipQuestIconSizeSetter(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	for _, button in pairs(tomQuest2.specialItemButtons.questsTooltip) do
		button:SetScale(value/button.height)
	end
	for _, button in pairs(tomQuest2.fakeItemButtons.questsTooltip) do
		button:SetScale(value/button.height)
	end
end

local questsTooltipOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["Quests tooltip"],
	get = qTOOptGetter,
	set = qTOOptSetter,
	args = {
		questsCounter = {
			type = "group",
			name = LCo["Quests counter"],
			set = questsTooltipLdbTextSetter,
			args = {
				showNumCompletedQuests = {
					name = LCo["Show number of completed quest"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				showNumQuests = {
					name = LCo["Show number of quests"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				showMaxQuest = {
					name = LCo["Show maximum quests number"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				colorizeQuestsCounter = {
					name = LCo["Color quests counter"],
					type = "toggle",
					width = "full",
					order = 4,
				},
			},
		},
		display = {
			type = "group",
			name = LCo["Display"],
			args = {
				hideCompletedObjective = {
					name = LCo["Hide completed quests objectives"],
					type = "select",
					values = hideObjectives,
					width = "full",
					order = 1,
				},
				hideZoneLevelRange = {
					name = LCo["Hide zone level range"],
					type = "toggle",
					set = hZLOptSetter,
					width = "full",
					order = 2,
				},
				hideQuestTag = {
					name = LCo["Hide quest tag"],
					type = "toggle",
					set = hQTOptSetter,
					width = "full",
					order = 3,
				},
			},
		},
		fonts = {
			type = "group",
			name = LCo["Tooltip fonts"],
			args = {
				zone = {
					type = "group",
					name = LCo["Zone"],
					set = questsTooltipCategoryFontSetter,
					inline = true,
					order = 1,
					args = {
						categoryFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						categoryFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						categoryFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				quests = {
					type = "group",
					name = LCo["Quests"],
					set = questsTooltipQuestFontSetter,
					inline = true,
					order = 2,
					args = {
						questFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				questsObjectives = {
					type = "group",
					name = LCo["Quests objectives"],
					set = questsTooltipQuestObjectiveFontSetter,
					inline = true,
					order = 3,
					args = {
						questObjectiveFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questObjectiveFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questObjectiveFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
			},
		},
		appearance = {
			type = "group",
			name = LCo["Tooltip appearance"],
			args = {
				bgTexture = {
					name = LCo["Background texture"],
					type = "select",
					dialogControl = "LSM30_Background",
					values = AceGUIWidgetLSMlists.background,
					set = questsTooltipBackdropSetter,
					order = 1,
				},
				borderTexture = {
					name = LCo["Border texture"],
					type = "select",
					dialogControl = "LSM30_Border",
					values = AceGUIWidgetLSMlists.border,
					set = questsTooltipBackdropSetter,
					order = 2,
				},
				borderWidth = {
					name = LCo["Border width"],
					type = "range",
					min = 1,
					max = 24,
					step = 1,
					set = questsTooltipBackdropSetter,
					order = 3,
				},
				borderInset = {
					name = LCo["Inset"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					set = questsTooltipBackdropSetter,
					order = 4,
				},
				scale = {
					name = LCo["Quests tooltip scale"],
					type="range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order = 5,
				},
				hasMaxHeight = {
					name = LCo["Set quests tooltip maximum height after which it will be scrollable"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				maxHeight = {
					name = LCo["Quests tooltip max height"],
					type = "range",
					min = 100,
					max = 2000,
					step = 5,
					width = "full",
					order = 7,
				},
				showCollapseButton = {
					name = LCo["Show collapse button"],
					type = "toggle",
					width = "full",
					order = 8,
				},
				collapseButtonSize = {
					name = LCo["Collapse button size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					set = questsTooltipCollapseButtonSizeSetter,
					width = "full",
					order = 9,
				},
				questsCellPadding = {
					name = LCo["Quests cell padding"],
					type="range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 10,
				},
				objectivesCellPadding = {
					name = LCo["Quests objectives cell padding"],
					type="range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 11,
				},
				maxWidth = {
					name = LCo["Quests Tooltip column max width"],
					type="range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 12,
				},
				minWidth = {
					name = LCo["Quests Tooltip column min width"],
					type="range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 13,
				},
				statusMaxWidth = {
					name = LCo["Quests Tooltip status column max width"],
					type="range",
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 14,
				},
				statusMinWidth = {
					name = LCo["Quests Tooltip status column min width"],
					type="range",
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 15,
				},
			},
		},
		colors = {
			type = "group",
			name = LCo["Colors"],
			args = {
				backDropColor = {
					name = LCo["Quests tooltip background color"],
					type = "color",
					hasAlpha = true,
					get = questsTooltipColorGetter,
					set = questsTooltipColorSetter,
					width = "full",
					order = 1,
				},
				borderColor = {
					name = LCo["Quests tooltip border color"],
					type = "color",
					hasAlpha = true,
					get = questsTooltipColorGetter,
					set = questsTooltipColorSetter,
					width = "full",
					order = 2,
				},
				colorizeCategoryName = {
					name = LCo["Color quests header name"],
					type = "toggle",
					set = setQuestsTooltipColors,
					width = "full",
					order = 3,
				},
				defaultCategoryNameColor = {
					name = LCo["Default header name color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 4,
				},
				colorizeCategoryLevel = {
					name = LCo["Color quests header level"],
					type = "toggle",
					set = setQuestsTooltipColors,
					width = "full",
					order = 5,
				},
				defaultCategoryLevelColor = {
					name = LCo["Default header level color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 6,
				},
				colorizeQuestName = {
					name = LCo["Color quests name"],
					type = "toggle",
					set = setQuestsTooltipColors,
					width = "full",
					order = 7,
				},
				defaultQuestNameColor = {
					name = LCo["Default quest name color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 8,
				},
				colorizeQuestLevel = {
					name = LCo["Color quests level"],
					type = "toggle",
					set = setQuestsTooltipColors,
					width = "full",
					order = 9,
				},
				defaultQuestLevelColor = {
					name = LCo["Default quest level color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 10,
				},
				colorizeQuestObjective = {
					name = LCo["Color quests objectives"],
					type = "toggle",
					set = setQuestsTooltipColors,
					width = "full",
					order = 11,
				},
				defaultQuestObjectiveColor = {
					name = LCo["Default quest objective color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 12,
				},
				colorizeQuestObjectiveCounter = {
					name = LCo["Color quests objectives counter"],
					type = "toggle",
					set = setQuestsTooltipColors,
					width = "full",
					order = 13,
				},
				defaultQuestObjectiveCounterColor = {
					name = LCo["Default quest objective counter color"],
					type = "toggle",
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 14,
				},
				defaultQuestDescriptionColor = {
					name = LCo["Default quest description color for quest without objective"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 15,
				},
			},
		},
		questsItems = {
			type = "group",
			name = LCo["Quests items"],
			args = {
				showQuestItemIcon = {
					name = LCo["Show special quest item icon behind quest name"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				questItemIconSize = {
					name = LCo["Quest item icon size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 2,
				},
			},
		},
		tooltipInteraction = {
			type = "group",
			name = LCo["Tooltip interaction"],
			args = {
				showHints = {
					name = LCo["Show interractivity hints at the bottom of the tooltip"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				overTooltip = {
					name = LCo["Display quest tooltip on mouse over"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				mouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 3,
				},
				mouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 4,
				},
				mouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 5,
				},
				mouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 6,
				},
				mouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 7,
				},
				mouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 8,
				},
				mouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 9,
				},
				mouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 10,
				},
				mouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 11,
				},
				mouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 12,
				},
				mouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 13,
				},
				mouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					order = 14,
				},
			},
		},
		ldbInteraction = {
			type = "group",
			name = LCo["Quests counter interaction"],
			get = optGetter,
			set = optSetter,
			args = {
				qToMouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 1,
				},
				qToMouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 2,
				},
				qToMouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 3,
				},
				qToMouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 4,
				},
				qToMouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 5,
				},
				qToMouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 6,
				},
				qToMouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 7,
				},
				qToMouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 8,
				},
				qToMouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 9,
				},
				qToMouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 10,
				},
				qToMouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 11,
				},
				qToMouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 12,
				},
			},
		},
	},
}

-----------------------------------
--- Quests Tracker option table ---
-----------------------------------
local questsAutoTrack = {[1] = LCo["Always"], [2] = LCo["Below max level"], [3] = LCo["Never"]}
local hideQuests = {[1] = LCo["When you've completed the quest"], [2] = LCo["When everyone in the party has completed the quest"], [3] = LCo["Never"]}
local function hideQuestsTrackerInteraction()
	if tomQuest2.db.profile.questsTracker.disableInteraction then
		return true
	else
		return false
	end
end
local function questsTrackerOptGetter(info)
	return tomQuest2.db.profile.questsTracker[info[#info]]
end
local function questsTrackerOptSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2:setQuestsTrackerLayout()
end
local function lockQuestsTrackerSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2.displayFrames["questsTracker"]:SetMovable(not value)
end
local function hZLOptSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for category in pairs(tomQuest2.questsCategories) do
		tomQuest2:updateQuestCategoryCell(category, "questsTracker")
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function hQTOptSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for qid in pairs(tomQuest2.quests) do
		tomQuest2:updateQuestTextCell(qid, "questsTracker")
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerCategoryFontSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2.cQTrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.categoryFont), tomQuest2.db.profile.questsTracker.categoryFontSize, tomQuest2.db.profile.questsTracker.categoryFontOutline)
	for category in pairs(tomQuest2.questsCategories) do
		tomQuest2.questCategoryCells.questsTracker[category].fs:SetFontObject(tomQuest2.cQTrFontTQ2)
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerQuestFontSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2.qQTrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.questFont), tomQuest2.db.profile.questsTracker.questFontSize, tomQuest2.db.profile.questsTracker.questFontOutline)
	for qid in pairs(tomQuest2.quests) do
		tomQuest2.questTextCells.questsTracker[qid].fs:SetFontObject(tomQuest2.qQTrFontTQ2)
		tomQuest2.questStatusCells.questsTracker[qid].fs:SetFontObject(tomQuest2.qQTrFontTQ2)
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerQuestObjectiveFontSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2.qoQTrFontTQ2:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.questObjectiveFont), tomQuest2.db.profile.questsTracker.questObjectiveFontSize, tomQuest2.db.profile.questsTracker.questObjectiveFontOutline)
	for qid, qoData in pairs(tomQuest2.questsObjectives) do
		for oid in pairs(qoData) do
			tomQuest2.questsObjectiveTextCells.questsTracker[qid][oid].fs:SetFontObject(tomQuest2.qoQTrFontTQ2)
			tomQuest2.questsObjectiveStatusCells.questsTracker[qid][oid].fs:SetFontObject(tomQuest2.qoQTrFontTQ2)
		end
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerBackdropSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2:setBackDrop("questsTracker")
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerCollapseButtonSizeSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for _, button in pairs(tomQuest2.categoryButtons.questsTracker) do
		button:SetScale(value/button.height)
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerColorGetter(info)
	return unpack(tomQuest2.db.profile.questsTracker[info[#info]])
end
local function questsTrackerColorSetter(info, r, g, b, a)
	tomQuest2.db.profile.questsTracker[info[#info]][1] = r
	tomQuest2.db.profile.questsTracker[info[#info]][2] = g
	tomQuest2.db.profile.questsTracker[info[#info]][3] = b
	tomQuest2.db.profile.questsTracker[info[#info]][4] = a
	tomQuest2:setQuestsTrackerLayout()
end
local function setquestsTrackerColors(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	if info[#info] == "colorizeCategoryName" or info[#info] == "colorizeCategoryLevel" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2:updateQuestCategoryCell(category, "questsTracker")
		end
	elseif info[#info] == "colorizeQuestName" or info[#info] == "colorizeQuestLevel" then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2:updateQuestTextCell(qid, "questsTracker")
		end
	elseif info[#info] == "colorizeQuestObjective" or info[#info] == "colorizeQuestObjectiveCounter" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2:updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTracker[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTracker.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTracker[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTracker.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor)
			end
		end
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerHexColorGetter(info)
	return tonumber(sub(tomQuest2.db.profile.questsTracker[info[#info]], 1, 2), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTracker[info[#info]], 3, 4), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTracker[info[#info]], 5, 6), 16) / 255
end
local function questsTrackerHexColorSetter(info, arg1, arg2, arg3)
	tomQuest2.db.profile.questsTracker[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
	if info[#info] == "defaultCategoryNameColor" or info[#info] == "defaultCategoryLevelColor" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2:updateQuestCategoryCell(category, "questsTracker")
		end
	elseif info[#info] == "defaultQuestNameColor" or info[#info] == "defaultQuestLevelColor" then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2:updateQuestTextCell(qid, "questsTracker")
		end
	elseif info[#info] == "defaultQuestObjectiveColor" or info[#info] == "defaultQuestObjectiveCounterColor" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2:updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTracker[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTracker.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTracker[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTracker.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor)
			end
		end
	end
	tomQuest2:setQuestsTrackerLayout()
end
local function questsTrackerQuestIconSizeSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for _, button in pairs(tomQuest2.specialItemButtons.questsTracker) do
		button:SetScale(value/button.height)
	end
	for _, button in pairs(tomQuest2.fakeItemButtons.questsTracker) do
		button:SetScale(value/button.height)
	end
	tomQuest2:setQuestsTrackerLayout()
end

local questsTrackerOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["Quests tracker"],
	get = questsTrackerOptGetter,
	set = questsTrackerOptSetter,
	args = {
		autotrack = {
			type = "group",
			name = LCo["Auto track"],
			set = optSetter,
			args = {
				onLogIn = {
					name = LCo["Track all quests on log in"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				onQuestAccepted = {
					name = LCo["Auto track quests when you accept them"],
					type = "select",
					values = questsAutoTrack,
					width = "full",
					order = 2,
				},
				onObjectiveProgress = {
					name = LCo["Auto track quests on objective progress"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				unTrackInactive = {
					name = LCo["Auto untrack inactive quests that were tracked on objective progress"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				unTrackDelay = {
					name = LCo["Auto untrack delay (in minutes)"],
					type = "range",
					min = 1,
					max = 30,
					step = 0.5,
					width = "full",
					order = 5,
				},
				onSubZone = {
					name = LCo["Auto track quests on sub zone change"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				onZone = {
					name = LCo["Auto track quests on zone change"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				onDaily = {
					name = LCo["Auto track daily quests"],
					type = "toggle",
					width = "full",
					order = 8,
				},
				onClass = {
					name = LCo["Auto track class quests"],
					type = "toggle",
					width = "full",
					order = 9,
				},
			},
		},
		display = {
			type = "group",
			name = LCo["Display"],
			args = {
				showByCategory = {
					name = LCo["Display quests by zone"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				autoCollapseCategory = {
					name = LCo["Automatically expand /collapse zone header on zone change"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				lockTracker = {
					name = LCo["Lock - unlock quests tracker"],
					type = "toggle",
					set = lockQuestsTrackerSetter,
					width = "full",
					order = 4,
				},
				toggleTracker = {
					name = LCo["Display tracker"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				hideTrackerInCombat = {
					name = LCo["Hide quests tracker in combat"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				hideCompletedQuest = {
					name = LCo["Hide completed quest"],
					type = "select",
					values = hideQuests,
					width = "full",
					order = 7,
				},
				hideCompletedObjective = {
					name = LCo["Hide completed quests objectives"],
					type = "select",
					values = hideObjectives,
					width = "full",
					order = 9,
				},
				onlyCurrentZone = {
					name = LCo["Hide quests that are not in the current zone"],
					type = "toggle",
					width = "full",
					order = 10,
				},
				limitQuestLevel = {
					name = LCo["Hide high level quests (You'll have to choose the maximum level difference)"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				maxLevelDiff = {
					name = LCo["Maximum level difference between the weighted quest level and your level"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					width = "full",
					order = 12,
				},
				hideZoneLevelRange = {
					name = LCo["Hide zone level range"],
					type = "toggle",
					set = hZLOptSetter,
					width = "full",
					order = 13,
				},
				hideQuestTag = {
					name = LCo["Hide quest tag"],
					type = "toggle",
					set = hQTOptSetter,
					width = "full",
					order = 14,
				},
			},
		},
		fonts = {
			type = "group",
			name = LCo["Tracker fonts"],
			args = {
				category = {
					type = "group",
					name = LCo["Zone"],
					set = questsTrackerHeadFontSetter,
					inline = true,
					order = 1,
					args = {
						categoryFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						categoryFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						categoryFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				quests = {
					type = "group",
					name = LCo["Quests"],
					set = questsTrackerQuestFontSetter,
					inline = true,
					order = 2,
					args = {
						questFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				questsObjectives = {
					type = "group",
					name = LCo["Quests objectives"],
					set = questsTrackerQuestObjectiveFontSetter,
					inline = true,
					order = 3,
					args = {
						questObjectiveFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questObjectiveFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questObjectiveFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
			},
		},
		appearance = {
			type = "group",
			name = LCo["Tracker appearance"],
			args = {
				bgTexture = {
					name = LCo["Background texture"],
					type = "select",
					dialogControl = "LSM30_Background",
					values = AceGUIWidgetLSMlists.background,
					set = questsTrackerBackdropSetter,
					order = 1,
				},
				borderTexture = {
					name = LCo["Border texture"],
					type = "select",
					dialogControl = "LSM30_Border",
					values = AceGUIWidgetLSMlists.border,
					set = questsTrackerBackdropSetter,
					order = 2,
				},
				borderWidth = {
					name = LCo["Border width"],
					type = "range",
					min = 1,
					max = 24,
					step = 1,
					set = questsTrackerBackdropSetter,
					order = 3,
				},
				borderInset = {
					name = LCo["Inset"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					set = questsTrackerBackdropSetter,
					order = 4,
				},
				scale = {
					name = LCo["Quests tracker scale"],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order = 5,
				},
				hasMaxHeight = {
					name = LCo["Set quests tracker maximum height after which it will be scrollable"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				maxHeight = {
					name = LCo["Quests tracker max height"],
					type = "range",
					min = 100,
					max = 2000,
					step = 5,
					width = "full",
					order = 7,
				},
				showCollapseButton = {
					name = LCo["Show collapse button"],
					type = "toggle",
					width = "full",
					order = 8,
				},
				collapseButtonSize = {
					name = LCo["Collapse button size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					set = questsTrackerCollapseButtonSizeSetter,
					width = "full",
					order = 9,
				},
				questsCellPadding = {
					name = LCo["Quests cell padding"],
					type = "range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 10,
				},
				objectivesCellPadding = {
					name = LCo["Quests objectives cell padding"],
					type = "range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 11,
				},
				maxWidth = {
					name = LCo["Quests tracker column max width"],
					type = "range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 13,
				},
				minWidth = {
					name = LCo["Quests tracker column min width"],
					type = "range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 14,
				},
				statusMaxWidth = {
					name = LCo["Quests tracker status column max width"],
					type="range",
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 15,
				},
				statusMinWidth = {
					name = LCo["Quests tracker status column min width"],
					type="range",
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 16,
				},
			},
		},
		color = {
			type = "group",
			name = LCo["Colors"],
			args = {
				backDropColor = {
					name = LCo["Quests tracker background color"],
					type = "color",
					hasAlpha = true,
					get = questsTrackerColorGetter,
					set = questsTrackerColorSetter,
					width = "full",
					order = 1,
				},
				borderColor = {
					name = LCo["Quests tracker border color"],
					type = "color",
					hasAlpha = true,
					get = questsTrackerColorGetter,
					set = questsTrackerColorSetter,
					width = "full",
					order = 2,
				},
				colorizeCategoryName = {
					name = LCo["Color quests header name"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				defaultCategoryNameColor = {
					name = LCo["Default header name color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 4,
				},
				colorizeCategoryLevel = {
					name = LCo["Color quests header level"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				defaultCategoryLevelColor = {
					name = LCo["Default header level color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 6,
				},
				colorizeQuestName = {
					name = LCo["Color quests name"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				defaultQuestNameColor = {
					name = LCo["Default quest name color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 8,
				},
				colorizeQuestLevel = {
					name = LCo["Color quests level"],
					type = "toggle",
					width = "full",
					order = 9,
				},
				defaultQuestLevelColor = {
					name = LCo["Default quest level color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 10,
				},
				colorizeQuestTimer = {
					name = LCo["Color quests timer"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				defaultQuestTimerColor = {
					name = LCo["Default quest timer color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 12,
				},
				colorizeQuestObjective = {
					name = LCo["Color quests objectives"],
					type = "toggle",
					width = "full",
					order = 13,
				},
				defaultQuestObjectiveColor = {
					name = LCo["Default quest objective color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 14,
				},
				colorizeQuestObjectiveCounter = {
					name = LCo["Color quests objectives counter"],
					type = "toggle",
					width = "full",
					order = 15,
				},
				defaultQuestObjectiveCounterColor = {
					name = LCo["Default quest objective counter color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 16,
				},
				defaultQuestDescriptionColor = {
					name = LCo["Default quest description color for quest without objective"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 17,
				},
			},
		},
		questsItems = {
			type = "group",
			name = LCo["Quests items"],
			args = {
				settings = {
					type = "group",
					name = LCo["Settings"],
					inline = true,
					order = 1,
					args = {
						showQuestItemIcon = {
							name = LCo["Show special quest item icon behind quest name"],
							type = "toggle",
							width = "full",
							order = 1,
						},
						questItemIconSize = {
							name = LCo["Quest item icon size"],
							type = "range",
							min = 6,
							max = 36,
							step = 1,
							width = "full",
							order = 2,
						},
						oorColor = {
							name = LCo["Out of range color"],
							type = "color",
							hasAlpha = false,
							get = questsTrackerColorGetter,
							set = questsTrackerColorSetter,
							width = "full",
							order = 3,
						},
						showItemsStartingQuest = {
							name = LCo["Show items starting a quest in the tracker"],
							type = "toggle",
							width = "full",
							order = 4,
						},
						showUsableQuestItemsNotAttached = {
							name = LCo["Show usable quest item not linked to a quest"],
							type = "toggle",
							width = "full",
							order = 5,
						},
						otherItemIconSize = {
							name = LCo["Other quest item icon size"],
							type = "range",
							min = 6,
							max = 36,
							step = 1,
							width = "full",
							order = 6,
						},
					},
				},
			},
		},
		trackerInteraction = {
			type = "group",
			name = LCo["Tracker interaction"],
			args = {
				disableInteraction = {
					type = "toggle",
					name = LCo["Completely disable tracker interaction, should not be used when display by category is used or if you wish to make it scrollable."],
					width = "full",
					order = 1,
				},
				overTooltip = {
					name = LCo["Display quest tooltip on mouse over"],
					type = "toggle",
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 2,
				},
				mouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 3,
				},
				mouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 4,
				},
				mouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 5,
				},
				mouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 6,
				},
				mouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 7,
				},
				mouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 8,
				},
				mouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 9,
				},
				mouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 10,
				},
				mouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 11,
				},
				mouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 12,
				},
				mouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 13,
				},
				mouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = tomQuest2.interactivityList,
					width = "full",
					hidden = hideQuestsTrackerInteraction,
					order = 14,
				},
			},
		},
	},
}

----------------------------------------
--- Sound Notifications option table ---
----------------------------------------
local soundNotificationsOptions = {
	type = "group",
	name = LCo["Sound notifications"],
	get = optGetter,
	set = optSetter,
	args = {
		soundObjectiveComplete = {
			name = LCo["Quest objective complete sound"],
			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
			width = "full",
			order = 1,
		},
		soundQuestComplete = {
			name = LCo["Quest complete sound"],
			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
			width = "full",
			order = 2,
		},
		soundQuestFailed = {
			name = LCo["Quest failed sound"],
			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
			width = "full",
			order = 3,
		},
	},
}

-----------------------------
--- tooltips option table ---
-----------------------------
local function mobTooltipOptSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	if not tomQuest2.moduleState["tooltips"] then return end
	tomQuest2:setMobTooltip()
end
local function itemTooltipOptSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	if not tomQuest2.moduleState["tooltips"] then return end
	tomQuest2:setItemTooltip()
end

local tooltipsOptions = {
	type = "group",
	name = LCo["Tooltips"],
	get = optGetter,
	args = {
		mobTooltip = {
			name = LCo["Mobs tooltip modification"],
			type = "toggle",
			width = "full",
			set = mobTooltipOptSetter,
			order = 1,
		},
		itemTooltip = {
			name = LCo["Item tooltip modification"],
			type = "toggle",
			width = "full",
			set = itemTooltipOptSetter,
			order = 2,
		},
	},
}

-------------------------
--- Make option panel ---
-------------------------
local profilesOptions = LibStub("AceDBOptions-3.0"):GetOptionsTable(tomQuest2.db)
ACR:RegisterOptionsTable("tomQuest2", options)
tomQuest2.mainOptionFrame = ACD:AddToBlizOptions("tomQuest2", "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_profiles", profilesOptions)
tomQuest2.profilesOptionFrame = ACD:AddToBlizOptions("tomQuest2_profiles", LCo["Profiles"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_achievementsTracker", achievementsTrackerOptions)
ACD:AddToBlizOptions("tomQuest2_achievementsTracker", LCo["Achievements tracker"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_announce", announceOptions)
ACD:AddToBlizOptions("tomQuest2_announce", LCo["Announce"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_dailyTooltip", dailyTooltipOptions)
ACD:AddToBlizOptions("tomQuest2_dailyTooltip", LCo["Daily tooltip"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_notifications", notificationsOptions)
ACD:AddToBlizOptions("tomQuest2_notifications", LCo["Notifications"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsGivers", questsGiversOptions)
ACD:AddToBlizOptions("tomQuest2_questsGivers", LCo["Quests givers"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsInformations", questsInformationsOptions)
ACD:AddToBlizOptions("tomQuest2_questsInformations", LCo["Information"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsTooltip", questsTooltipOptions)
ACD:AddToBlizOptions("tomQuest2_questsTooltip", LCo["Quests tooltip"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsTracker", questsTrackerOptions)
ACD:AddToBlizOptions("tomQuest2_questsTracker", LCo["Quests tracker"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_soundNotifications", soundNotificationsOptions)
ACD:AddToBlizOptions("tomQuest2_soundNotifications", LCo["Sound notifications"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_tooltips", tooltipsOptions)
ACD:AddToBlizOptions("tomQuest2_tooltips", LCo["Tooltips"], "tomQuest2")